#####################################################################################################################
# Copyright IBM
#
# This PERL script was created by Shirley Dango of Rational Software.
# It was created on November 18, 2002 for the RATLC production database as part of the CC/CQ/UCM Integration project.
# The purpose is to update the promotion level of the Baseline record in the RATLC production database.
# The script will search for the baseline record within the RATLC production database.
# The user must provide the user login id, password, database name (RATLC), database set name,
# the baseline record to update, the project vob of the baseline, and the new promotion level information.
# 
#
# Updated by: Shirley Dango
# Updated on: April 21, 2003
# Updated January, 2008 by Katie Carpenter
######################################################################################################################
# 
#This script utilized the ClearQuest COM API and the ClearCase cleartool commands.
# 
# Variable used in this script are as followings:
#	user		The login id of the ClearQuest user
#	pw			The password of the ClearQuest user
#	dbname		The schema repository connection to the database
#	dbset		The user database connection
#	baseline	The name of the baseline record
#	pvob		The name of the project vob for this baseline
#	get_promotionlevel	The subrountine finds the promotion level for the baseline
#	logfile			The path where the log file will be sent
#	test			This is a flag that can be set for testing file inputs
#	help			This is a flag for the help
#	EDIT_BUILD_ACTION	This variable will set a session variable for ClearQuest access.
#######################################################################################################################
#
# Change Log:
# Flag      Item                Date        Programmer         Comments
# thl1   RATLC01087702       06/28/2005      THL               UCM Integration Scripts should use ratlperl not cqperl
# thl2   RATLC00475437       10/18/2005      THL               the baseline create perl module needs to also check the returned 
#                                                              status from CQ API calls.
# thl3   RATLC00475646       10/19/2005      THL               Need to check for "\" on pvob parm
#        RATLC00396394	     1/10/2008       katherns          Update obsolete field
#
#######################################################################################################################
#!ratlperl -l -w
# Changed above line to read ratlperl from cqperl thl1
use strict; 
use vars qw($Bin);
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Long;
use File::Basename;
use Carp;
use FileHandle;
use File::Spec;  
use CQPerlExt;
use Env;
use LWP::Simple;
use POSIX;
my $promotion_level;
my $slashvob;
my $noslashvob;
my ($entity);
my ($failure);
my ($status,$resultSetStatus); #thl2
my $progname = basename $0;
use subs qw(get_promotionlevel main);

# Unbuffer STDERR and STDOUT
select STDERR;  
$| = 1;
select STDOUT;
$| = 1;

#Define Logger parameters
use Logger qw(CONSOLE_ON CONSOLE_OFF CONSOLE_TERSE);
my $logger = Logger->new(); # the logger element

# Create the utility object
use UtilLib;
my $ut = UtilLib->new();
my $log_time = $ut->ut_file_name_date_time_stamp();

my $progname = basename $0;

my $result = main();

if ($result) {
    exit(0); # DOS Success Code
} else {
    exit(1); # DOS Error Code
}

#####
# Subroutine to display the usage
sub Usage {
    print "

    Utility to logon and create a session using the CQ API
    Usage:
        $progname -user <username>
         -pw <password>
         -dbname <database name>
         -dbset <database set>
         -pvob <pvob>
         -baseline <baseline> 
		 -logfile <logfile>
		 -test
		 -help
";
return 0;
}

sub main {
####
# Define the input parameters
my @opts = qw(
    user=s		
    pw=s		
    dbname=s		
    dbset=s		
    pvob=s		
    baseline=s	
    logfile=s
    test
    help
);

my %opts = (
    user => undef,
    pw => undef,
    dbname => undef,
    dbset => undef,
    pvob => undef,
    baseline => undef,
    logfile => undef,
);


####
# Get the input parameters
unless (GetOptions(\%opts, @opts)) {
     	$logger->log_print_message("Printing Usage to the command line:");	
      	my $retval = &Usage;
      	return 0;
}

#Define the LogFile
if ($opts{logfile}) {
	my $retval;
	my $logdir = dirname($opts{logfile});
	if ( ! -e $logdir) {
		$retval = $ut->ut_create_directory_tree($logdir);
		if ( ! $retval) {
            		$logger->log_print_error( __LINE__ . "Could not create the log file directory: $logdir");
            		$logger->log_print_error( __LINE__ . "Script execution will continue without it.");
        	}
	}
	$retval = $logger->log_set_file_on($opts{logfile});
	if ( ! $retval) {
        	$logger->log_print_error( __LINE__ . "Could not open the log file: $opts{logfile}.");
        	$logger->log_print_error( __LINE__ . "Script execution will continue without it.");
   	 }
	 $logger->log_set_console(CONSOLE_OFF);
	$logger->log_print_message("Logging started at $log_time"); 
}	 



####
# Display the help usage
if ($opts{help}) {
	print "-user		The login id of the ClearQuest user.\n";
	print "-pw			The password of the ClearQuest user.\n";
	print "-dbname		The schema repository connection to the database.\n";
	print "-dbset		The user database connection.\n";
	print "-pvob		The name of the project vob.\n";
	print "-baseline	The name of the baseline.\n";
	print "-logfile		The path where the log file will be sent.\n";
	print "-test		This is a flag that can be set for testing file inputs.\n";
	print "-help		This is a flag for the help.\n";
    exit;
}
# Display test inputs to log file 
if ($opts{test}){
	$logger->log_print_message("main: Defined user inputs:");
	$logger->log_print_message("main: Defined user login is $opts{user}.");
	$logger->log_print_message("main: Defined dbset is $opts{dbset}.");
	$logger->log_print_message("main: Defined database name is $opts{dbname}.");
	$logger->log_print_message("main: Defined baseline name is $opts{baseline}.");
	$logger->log_print_message("main: Defined Project VOB is $opts{pvob}.");
	if ($opts{logfile}){ $logger->log_print_message("Defined logfile is $opts{logfile}.");}
	return 1;
}

####
# End program if any of the parameters are not defined
if (!defined $opts{user}
  or !defined $opts{pw}
  or !defined $opts{dbname}
  or !defined $opts{dbset}
  or !defined $opts{pvob}
  or !defined $opts{baseline}) {
  $logger->log_print_message("Printing Usage to the command line:");
  my $retval = &Usage();
  return 0;
}
;


####
#### Start a CQ session
my $session = CQPerlExt::CQSession_Build();
# Set session variable 
#$session->SetNameValue("EDIT_BUILD_ACTION", "ACCESS_OK");
#### Logon to the specified database

eval {$session->UserLogon("$opts{user}", "$opts{pw}", "$opts{dbname}", "$opts{dbset}")};

if ($@) {
	$logger->log_print_error(__LINE__ . "main: Login to $opts{dbname} failed: $@"); 
	return 0;
}

$logger->log_print_success("main: Login to $opts{dbname} succeeded.");
#

#### Set up vobs
##Passing in the vob to CQ requires no slashes on the vob, passing in to CC requires a slash on the vob
sub slash_noslash {
	my $pvob = shift;
	my $slashvob;
	my $noslashvob;
	if ($pvob !~ /[\/\\]/) {
	  $slashvob = '\\' .$pvob;
	  $noslashvob = $pvob;
	}
	else{
	  $slashvob = $pvob;
	  $noslashvob = $pvob ;
	  $noslashvob=~ s/\\//g; 
	}
	return ($slashvob, $noslashvob);
}
($slashvob, $noslashvob ) = &slash_noslash($opts{pvob});




#### Get record from ClearQuest

#### Build a query to find the Baseline record

my $queryDef = $session->BuildQuery("ALMBaseline");
my $failure = $queryDef->BuildField("Name");

my $operator = $queryDef->BuildFilterOperator($CQPerlExt::CQ_BOOL_OP_AND);
$operator->BuildFilter("Name", $CQPerlExt::CQ_COMP_OP_EQ, ["$opts{baseline}"]);
$operator->BuildFilter("PVOB_OrLocation", $CQPerlExt::CQ_COMP_OP_EQ, ["$noslashvob"]); #021703

my $resultSet;

eval {$resultSet = $session->BuildResultSet($queryDef)};
if ($@) {
	$logger->log_print_error(__LINE__ . "main: Unable to build ResultSet from Query:"); 
	$logger->log_print_error(__LINE__ . "main: $@"); 
	return 0;
}

$logger->log_print_message("main: Built resultset from query.");

$resultSet->Execute;

$resultSetStatus = $resultSet->MoveNext;  # thl2
if ($resultSetStatus != $CQPerlExt::CQ_SUCCESS){   #thl2
	$logger->log_print_error(__LINE__ . "main: Baseline record not found.");
	$logger->log_print_error(__LINE__ . "main: Please create a new Baseline record using create_baseline_record.pl.");
	$logger->log_print_success("main: Finished update_baseline_record script.");
        CQPerlExt::CQSession_Unbuild($session);
        return 0;
}

#### Print out the resultset
while($resultSetStatus == $CQPerlExt::CQ_SUCCESS) { #thl2
	my $foundrecord = $resultSet->GetColumnValue(1);
	print ("$foundrecord\n");
	$logger->log_print_message("main: Found record: $foundrecord.");
	
	####Get record 
	my $record1;
	eval {$record1 = $session->GetEntity("ALMBaseline", $noslashvob.' '.$foundrecord)};
	if ($@) {  
		$logger->log_print_error(__LINE__ . "main: No records found in database to match query"); 
		$logger->log_print_error(__LINE__ . "main: Please run create_baseline_record.pl to create a new baseline record"); 
                return 0;
	}
	
	## Find the promotion level of the baseline
	$promotion_level = get_promotionlevel(baseline => $opts{baseline}, pvob => $slashvob);#changed
	if (!$promotion_level) {
		$logger->log_print_error(__LINE__ . "main: Unable to retrieve promotion level"); 
		$logger->log_print_error(__LINE__ . "main: Exiting without updating baseline record"); 
	}	
	
	## Edit the record to add the new promotion level
        $status = "" ;                                             #thl2
	eval {$session->EditEntity($record1,"modify")};  
	if ($@) {                                   
		$logger->log_print_error(__LINE__ . "main: Could not modify record $record1");  
		$logger->log_print_error(__LINE__ . "main: $@");
                return 0;
	}
	
	my $failure = $record1->SetFieldValue("PromotionLevel", $promotion_level);


	##If baseline is obsolete in CC, it should be obsolete in CQ
	my $cmd = qq(cleartool lslock baseline:"$opts{baseline}\@$slashvob"); 
	my ($retval, $output) = $ut->ut_shell($cmd);
        unless (defined $retval){		
		$logger->log_print_error(__LINE__ ."Failed to retrieve lock information");
		$logger->log_print_error(__LINE__ ."Baseline cannot be updated with lock information");
		
        }
	
	if($output =~ m/locked/i) {

		if($output =~ m/obsolete/i) {
		  my $failure = $record1->SetFieldValue("Obsolete", "Yes");			
	          $logger->log_print_message("Baseline set to obsolete. Continue to validate.");
	        }
		else {
		  $logger->log_print_message("Baseline is locked.");
		}
	  
	}
	else {
		my $failure = $record1->SetFieldValue("Obsolete", "No");		
		$logger->log_print_message("Baseline is set to not obsolete.  Continue to validate.");
	}

        
	#Validate the Baseline Record
	$logger->log_print_message("main: Validating the Baseline record");
        $status = "" ;                                             #thl2
	eval {$status = $record1->Validate()};
	if (($@) || ($status)) {                                   #thl2
		$logger->log_print_error(__LINE__ . "main: Could not validate record $record1: $@"); #thl2
                $logger->log_print_error(__LINE__ . "Returned Status is $status") ;  #thl2
		return 0;
	}
	$status = "" ; #thl2
	
	# Committing the Baseline Record
	$logger->log_print_message("main: Committing the Baseline record");
	eval {$status = $record1->Commit()}; #thl2
	if (($@) || ($status)) {                                   #thl2
		$logger->log_print_error(__LINE__ . "main: Could not commit record $record1: $@"); #thl2
                $logger->log_print_error(__LINE__ . "Returned Status is $status") ;  #thl2
		return 0;
	}
	$record1 = undef;

	$logger->log_print_success("main: Baseline Record has been updated");
	CQPerlExt::CQSession_Unbuild($session);
	return 1;	
	$resultSet->MoveNext ;  
} # end of loop

# Write to log file 
$logger->log_print_success("main: Finished update_baseline_record script.");
CQPerlExt::CQSession_Unbuild($session);
return 1;
}
# Find Promotion Level for the Baseline Record  
#
sub get_promotionlevel { 
	my %opts = @_;
	my $cmd = qq(cleartool lsbl -l "$opts{baseline}\@$slashvob"); 
	my ($retval, $output) = $ut->ut_shell($cmd);
        unless (defined $retval){
		$logger->log_print_error(__LINE__ . " Cleartool command: $cmd"); 
		$logger->log_print_error(__LINE__ . "failed to retrieve Promotion Level information");
		$logger->log_print_error(__LINE__ . " Baseline cannot be created without Promotion Level information");
		return 0;
        }
	$output =~ /promotion level: (.*?)$/ms;
	my $plevel = $1;
	chomp $plevel;
	$logger->log_print_success(" Promotion level is $plevel");
	return $plevel;
} 
# end of script
